
/* Chrysalide - Outil d'analyse de fichiers binaires
 * core.c - chargement et déchargement des mécanismes internes de l'architecture ARMv7
 *
 * Copyright (C) 2016-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "core.h"


#include "register.h"
#include "operands/coproc.h"
#include "operands/estate.h"
#include "operands/limitation.h"
#include "operands/maccess.h"
#include "operands/offset.h"
#include "operands/reglist.h"
#include "operands/rotation.h"
#include "operands/shift.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Met en place les mécanismes internes de l'architecture ARMv7.*
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool init_armv7_core(void)
{
    bool result;                            /* Bilan à renvoyer            */

    result = init_armv7_register_sharing();

    if (result)
        result = init_armv7_coproc_operand_sharing();

    if (result)
        result = init_armv7_endian_operand_sharing();

    if (result)
        result = init_armv7_limitation_operand_sharing();

    if (result)
        result = init_armv7_maccess_operand_sharing();

    if (result)
        result = init_armv7_offset_operand_sharing();

    if (result)
        result = init_armv7_reglist_operand_sharing();

    if (result)
        result = init_armv7_rotation_operand_sharing();

    if (result)
        result = init_armv7_shift_operand_sharing();

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Imprime des statistiques quant aux partages dans l'archi.    *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/
#ifdef DEBUG_DUMP_STATS
void dump_armv7_share_stats(void)
{
    dump_armv7_register_share_stats();

    dump_armv7_coproc_operand_share_stats();
    dump_armv7_endian_operand_share_stats();
    dump_armv7_limitation_operand_share_stats();
    dump_armv7_maccess_operand_share_stats();
    dump_armv7_offset_operand_share_stats();
    dump_armv7_reglist_operand_share_stats();
    dump_armv7_rotation_operand_share_stats();
    dump_armv7_shift_operand_share_stats();

}
#endif


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Supprime les mécanismes internes de l'architecture ARMv7.    *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void exit_armv7_core(void)
{
    exit_armv7_register_sharing();

    exit_armv7_coproc_operand_sharing();
    exit_armv7_endian_operand_sharing();
    exit_armv7_limitation_operand_sharing();
    exit_armv7_maccess_operand_sharing();
    exit_armv7_offset_operand_sharing();
    exit_armv7_reglist_operand_sharing();
    exit_armv7_rotation_operand_sharing();
    exit_armv7_shift_operand_sharing();

}
