
/* Chrysalide - Outil d'analyse de fichiers binaires
 * endian.c - décallages de valeurs
 *
 * Copyright (C) 2016-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "estate.h"


#include "../../../operand-int.h"
#include "../../../sharing/manager.h"
#include "../../../../common/sort.h"



/* --------------------- MANIPULATION D'OPERANDES INDIVIDUELLES --------------------- */


/* Définition d'un opérande affichant le choix d'un boutisme (instance) */
struct _GArmV7EndianOperand
{
    GArchOperand parent;                    /* Instance parente            */

    bool big;                               /* Grand boutisme à afficher ? */

};


/* Définition d'un opérande affichant le choix d'un boutisme (classe) */
struct _GArmV7EndianOperandClass
{
    GArchOperandClass parent;               /* Classe parente              */

};


/* Initialise la classe des affichages de boutisme. */
static void g_armv7_endian_operand_class_init(GArmV7EndianOperandClass *);

/* Initialise une instance d'affichage de boutisme. */
static void g_armv7_endian_operand_init(GArmV7EndianOperand *);

/* Supprime toutes les références externes. */
static void g_armv7_endian_operand_dispose(GArmV7EndianOperand *);

/* Procède à la libération totale de la mémoire. */
static void g_armv7_endian_operand_finalize(GArmV7EndianOperand *);

/* Réalise une copie minimale d'un contenu partagé. */
static void g_armv7_endian_operand_define_template(const GArmV7EndianOperand *, GArmV7EndianOperand *);

/* Compare un opérande avec un autre. */
static int g_armv7_endian_operand_compare(const GArmV7EndianOperand *, const GArmV7EndianOperand *);

/* Traduit un opérande en version humainement lisible. */
static void g_armv7_endian_operand_print(const GArmV7EndianOperand *, GBufferLine *, AsmSyntax);



/* -------------------------- PARTAGES DE CONTENUS UNIQUES -------------------------- */


/* Gestionnaire des partages d'instances */
static GShareManager *_armv7_endian_operand_manager = NULL;


/* Fournit le gestionnaire de partages attribué à un type. */
static GShareManager *get_armv7_endian_operand_share_manager(void);



/* ---------------------------------------------------------------------------------- */
/*                       MANIPULATION D'OPERANDES INDIVIDUELLES                       */
/* ---------------------------------------------------------------------------------- */


/* Indique le type défini par la GLib pour une endian de domaine et d'accès. */
G_DEFINE_TYPE(GArmV7EndianOperand, g_armv7_endian_operand, G_TYPE_ARCH_OPERAND);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des affichages de boutisme.             *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_armv7_endian_operand_class_init(GArmV7EndianOperandClass *klass)
{
    GObjectClass *object;                   /* Autre version de la classe  */
    GArchOperandClass *operand;             /* Version de classe parente   */

    object = G_OBJECT_CLASS(klass);
    operand = G_ARCH_OPERAND_CLASS(klass);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_armv7_endian_operand_dispose;
    object->finalize = (GObjectFinalizeFunc)g_armv7_endian_operand_finalize;

    operand->get_manager = (get_operand_manager_fc)get_armv7_endian_operand_share_manager;

    operand->apply_template = (apply_operand_template_fc)NULL;
    operand->define_template = (define_operand_template_fc)g_armv7_endian_operand_define_template;
    operand->free_template = (free_operand_template_fc)NULL;

    operand->compare = (operand_compare_fc)g_armv7_endian_operand_compare;
    operand->print = (operand_print_fc)g_armv7_endian_operand_print;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = instance à initialiser.                            *
*                                                                             *
*  Description : Initialise une instance d'affichage de boutisme.             *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_armv7_endian_operand_init(GArmV7EndianOperand *operand)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = instance d'objet GLib à traiter.                   *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_armv7_endian_operand_dispose(GArmV7EndianOperand *operand)
{
    G_OBJECT_CLASS(g_armv7_endian_operand_parent_class)->dispose(G_OBJECT(operand));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = instance d'objet GLib à traiter.                   *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_armv7_endian_operand_finalize(GArmV7EndianOperand *operand)
{
    G_OBJECT_CLASS(g_armv7_endian_operand_parent_class)->finalize(G_OBJECT(operand));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand  = objet partagé à consulter.                        *
*                template = informations à retrouver intégralement.           *
*                                                                             *
*  Description : Réalise une copie minimale d'un contenu partagé.             *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_armv7_endian_operand_define_template(const GArmV7EndianOperand *operand, GArmV7EndianOperand *template)
{
    template->big = operand->big;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : a = premier opérande à consulter.                            *
*                b = second opérande à consulter.                             *
*                                                                             *
*  Description : Compare un opérande avec un autre.                           *
*                                                                             *
*  Retour      : Bilan de la comparaison.                                     *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static int g_armv7_endian_operand_compare(const GArmV7EndianOperand *a, const GArmV7EndianOperand *b)
{
    int result;                             /* Bilan à faire remonter      */

    result = sort_boolean(a->big, b->big);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à traiter.                                *
*                line    = ligne tampon où imprimer l'opérande donné.         *
*                syntax  = type de représentation demandée.                   *
*                                                                             *
*  Description : Traduit un opérande en version humainement lisible.          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_armv7_endian_operand_print(const GArmV7EndianOperand *operand, GBufferLine *line, AsmSyntax syntax)
{
    if (operand->big)
        g_buffer_line_append_text(line, BLC_ASSEMBLY, "BE", 2, RTT_KEY_WORD, NULL);
    else
        g_buffer_line_append_text(line, BLC_ASSEMBLY, "LE", 2, RTT_KEY_WORD, NULL);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : big = indication sur le boutisme à représenter.              *
*                                                                             *
*  Description : Crée une représentation de boutisme ARMv7.                   *
*                                                                             *
*  Retour      : Opérande mis en place.                                       *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArchOperand *g_armv7_endian_operand_new(bool big)
{
    GSharedInstance *result;                /* Structure à retourner       */
    GArmV7EndianOperand template;           /* Transport d'informations    */

    template.big = big;

    result = g_share_manager_build(_armv7_endian_operand_manager, (GSharedInstance *)&template);

    return G_ARCH_OPERAND(result);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à consulter.                              *
*                                                                             *
*  Description : Indique le type de boutisme représenté.                      *
*                                                                             *
*  Retour      : Type de boutisme.                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_armv7_endian_operand_is_big_endian(const GArmV7EndianOperand *operand)
{
    return operand->big;

}



/* ---------------------------------------------------------------------------------- */
/*                            PARTAGES DE CONTENUS UNIQUES                            */
/* ---------------------------------------------------------------------------------- */


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit le gestionnaire de partages attribué à un type.      *
*                                                                             *
*  Retour      : Gestionnaire de partages en place.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static GShareManager *get_armv7_endian_operand_share_manager(void)
{
    return _armv7_endian_operand_manager;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Met en place les mécanismes de partage des états de boutisme.*
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool init_armv7_endian_operand_sharing(void)
{
    _armv7_endian_operand_manager = g_share_manager_new(G_TYPE_ARMV7_ENDIAN_OPERAND);

    return true;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Imprime des statistiques quant aux partages dans l'archi.    *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/
#ifdef DEBUG_DUMP_STATS
void dump_armv7_endian_operand_share_stats(void)
{
    g_share_manager_dump_stats(_armv7_endian_operand_manager);

}
#endif


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Supprime les mécanismes de partage des états de boutisme.    *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void exit_armv7_endian_operand_sharing(void)
{
    g_object_unref(G_OBJECT(_armv7_endian_operand_manager));

}
